# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
import torch.nn as nn
import torch    
class Model(nn.Module):   
    def __init__(self, encoder, config, tokenizer, args):
        super(Model, self).__init__()
        self.encoder = encoder
        self.config=config
        self.tokenizer=tokenizer
        self.args=args
        
        """Inspired from https://huggingface.co/transformers/v1.1.0/_modules/pytorch_transformers/modeling_roberta.html"""
        self.dense = nn.Linear(self.encoder.config.hidden_size, self.encoder.config.hidden_size)
        self.dropout = nn.Dropout(self.encoder.config.hidden_dropout_prob)
        self.out_proj = nn.Linear(self.encoder.config.hidden_size, 2) 
      
    def forward(self, mode="fast", bimodal_inputs=None,
                code_inputs=None, nl_inputs=None):

        if mode == "fast":
            if code_inputs is not None:
                return self.encoder(code_inputs, attention_mask=code_inputs.ne(1))[1]
            if nl_inputs is not None:
                return self.encoder(nl_inputs, attention_mask=nl_inputs.ne(1))[1]
        if mode == "slow":

            x = self.encoder(bimodal_inputs,attention_mask=bimodal_inputs.ne(1))[1]
            """
            need to check if this is cls
            tanh and dropout can be avoided - just one projection layer (maybe)
            """
            x = self.dropout(x)
            x = self.dense(x)
            x = torch.tanh(x)
            x = self.dropout(x)
            
            x = self.out_proj(x)

        return x
        